data "aws_caller_identity" "current" {}

data "aws_region" "current" {}

resource "aws_security_group" "pritunl" {
  name        = "${var.environment}-pritunl-sg"
  description = "${var.environment} pritunl vpn"
  vpc_id      = data.terraform_remote_state.vpc.outputs.vpc.vpc_id

  ingress {
    from_port = 0
    to_port   = 0
    protocol  = -1
    self      = true
  }

  ingress {
    from_port   = -1
    to_port     = -1
    protocol    = "icmp"
    cidr_blocks = var.pritunl_info.trusted_cidr_ranges
    description = "trusted_cidr_ranges ping"
  }

  ingress {
    from_port   = 22
    to_port     = 22
    protocol    = "tcp"
    cidr_blocks = var.pritunl_info.trusted_cidr_ranges
    description = "trusted_cidr_ranges ssh"
  }

  ingress {
    from_port   = 80
    to_port     = 80
    protocol    = "tcp"
    cidr_blocks = ["0.0.0.0/0"]
    description = "Pritunl Web UI - Required by letsencrypt"
  }

  ingress {
    from_port   = 443
    to_port     = 443
    protocol    = "tcp"
    cidr_blocks = flatten([[data.terraform_remote_state.vpc.outputs.vpc.vpc_cidr_block], var.pritunl_info.trusted_cidr_ranges])
    description = "Pritunl Web UI"
  }

  ingress {
    from_port   = 11500
    to_port     = 11500
    protocol    = "udp"
    cidr_blocks = ["0.0.0.0/0"]
    description = "VPN client connections"
  }

  egress {
    from_port   = 0
    to_port     = 0
    protocol    = -1
    cidr_blocks = ["0.0.0.0/0"]
  }

  tags = merge(local.required_tags, var.tags)
}

## Pritunl recommends CentOS due to SELinux functionality
## https://wiki.centos.org/Cloud/AWS
data "aws_ami" "centos" {
  owners      = ["679593333241"]
  most_recent = true

  filter {
    name   = "name"
    values = ["CentOS Linux 7 x86_64 HVM EBS *"]
  }

  filter {
    name   = "architecture"
    values = ["x86_64"]
  }

  filter {
    name   = "root-device-type"
    values = ["ebs"]
  }

  filter {
    name   = "virtualization-type"
    values = ["hvm"]
  }
}

data "template_file" "cloud_init" {
  template = file("cloud-init.yml.tpl")
  vars = {
    environment                       = var.environment
    pritunl_mongodb_connection_format = var.pritunl_info.pritunl_mongodb_connection_format
    pritunl_mongodb_username          = var.pritunl_info.pritunl_mongodb_username
    pritunl_mongodb_connection        = var.pritunl_info.pritunl_mongodb_connection
  }
}

resource "aws_launch_configuration" "pritunl" {
  name_prefix          = "pritunl"
  iam_instance_profile = aws_iam_instance_profile.pritunl.name
  image_id             = data.aws_ami.centos.id
  instance_type        = var.pritunl_info.instance_type
  key_name             = var.pritunl_info.keypair
  security_groups      = [aws_security_group.pritunl.id]
  user_data            = data.template_file.cloud_init.rendered

  root_block_device {
    delete_on_termination = true
  }

  lifecycle {
    create_before_destroy = true
  }
}

resource "aws_autoscaling_group" "pritunl" {
  name_prefix               = "pritunl-${aws_launch_configuration.pritunl.name}-"
  max_size                  = 1
  min_size                  = 0
  desired_capacity          = 1
  health_check_grace_period = 300
  health_check_type         = "EC2"
  launch_configuration      = aws_launch_configuration.pritunl.name
  vpc_zone_identifier       = data.terraform_remote_state.vpc.outputs.vpc.public_subnets

  target_group_arns = [
    aws_lb_target_group.pritunl-udp.arn,
    aws_lb_target_group.pritunl-http.arn,
    aws_lb_target_group.pritunl-https.arn
  ]

  lifecycle {
    create_before_destroy = true
  }

  tag {
    key                 = "Name"
    value               = "pritunl"
    propagate_at_launch = true
  }

  dynamic "tag" {
    for_each = merge(local.required_tags, var.tags)

    content {
      key                 = tag.key
      value               = tag.value
      propagate_at_launch = true
    }
  }
}
